import { addDialog } from '@/components/BaseDialog/index';
import CategoryDialog from '@/views/financial/account-bill/category/category-dialog.vue';
import { useCategoryStore } from '@/store/financial/category';
import { h, ref } from 'vue';
import { message, messageBox } from '@/utils/message';
import type { FormItemProps } from '@/views/financial/account-bill/category/utils/types';
import { $t } from '@/plugins/i18n';
import DeleteBatchDialog from '@/components/Table/DeleteBatchDialog.vue';

export const formRef = ref();
// 删除ids
export const deleteIds = ref([]);
export const switchLoadMap = ref({});
const categoryStore = useCategoryStore();

/** 搜索初始化分类信息 */
export async function onSearch() {
	categoryStore.loading = true;
	await categoryStore.getCategoryList();
	categoryStore.loading = false;
}

/** 添加分类信息 */
export function onAdd() {
	addDialog({
		title: `${$t('addNew')}${$t('category')}`,
		width: '30%',
		props: {
			formInline: {
				userId: undefined,
				categoryName: undefined,
				isBuiltin: false,
			},
		},
		draggable: true,
		fullscreenIcon: true,
		closeOnClickModal: false,
		contentRenderer: () => h(CategoryDialog, { ref: formRef }),
		beforeSure: (done, { options }) => {
			const form = options.props.formInline as FormItemProps;
			formRef.value.formRef.validate(async (valid: any) => {
				if (!valid) return;

				const result = await categoryStore.addCategory(form);
				if (!result) return;
				done();
				await onSearch();
			});
		},
	});
}

/** 更新分类信息 */
export function onUpdate(row: any) {
	addDialog({
		title: `${$t('modify')}${$t('category')}`,
		width: '30%',
		props: {
			formInline: {
				userId: row.userId,
				categoryName: row.categoryName,
				isBuiltin: row.isBuiltin,
			},
		},
		draggable: true,
		fullscreenIcon: true,
		closeOnClickModal: false,
		contentRenderer: () => h(CategoryDialog, { ref: formRef }),
		beforeSure: (done, { options }) => {
			const form = options.props.formInline as FormItemProps;
			formRef.value.formRef.validate(async (valid: any) => {
				if (!valid) return;

				const result = await categoryStore.updateCategory({ ...form, id: row.id });
				if (!result) return;
				done();
				await onSearch();
			});
		},
	});
}

/** 删除分类信息 */
export const onDelete = async (row: any) => {
	const id = row.id;

	// 是否确认删除
	const result = await messageBox({
		title: $t('confirmDelete'),
		showMessage: false,
		confirmMessage: undefined,
		cancelMessage: $t('cancel_delete'),
	});
	if (!result) return;

	// 删除数据
	await categoryStore.deleteCategory([id]);
	await onSearch();
};

/** 批量删除 */
export const onDeleteBatch = async () => {
	const ids = deleteIds.value;
	const formDeletedBatchRef = ref();

	addDialog({
		title: $t('deleteBatchTip'),
		width: '30%',
		props: { formInline: { confirmText: '' } },
		draggable: true,
		fullscreenIcon: true,
		closeOnClickModal: false,
		contentRenderer: () => h(DeleteBatchDialog, { ref: formDeletedBatchRef }),
		beforeSure: (done, { options }) => {
			formDeletedBatchRef.value.formDeletedBatchRef.validate(async (valid: any) => {
				if (!valid) return;

				const text = options.props.formInline.confirmText.toLowerCase();
				if (text === 'yes' || text === 'y') {
					// 删除数据
					await categoryStore.deleteCategory(ids);
					await onSearch();

					done();
				} else message($t('deleteBatchTip'), { type: 'warning' });
			});
		},
	});
};

/** 修改是否是系统内置字段 */
export const onUpdateIsBuiltin = async (row: any, index: number) => {
	// 点击时开始loading加载
	switchLoadMap.value[index] = Object.assign({}, switchLoadMap.value[index], {
		loading: true,
	});

	// 是否确认修改显示状态
	const confirm = await messageBox({
		title: $t('confirm_update_status'),
		showMessage: false,
		confirmMessage: undefined,
		cancelMessage: $t('cancel'),
	});

	// 取消修改
	if (!confirm) {
		row.isBuiltin = !row.isBuiltin;
		switchLoadMap.value[index] = Object.assign({}, switchLoadMap.value[index], {
			loading: false,
		});
		return;
	}

	// 确认修改
	const data = {
		id: row.id,
		userId: row.userId,
		categoryName: row.categoryName,
		isBuiltin: row.isBuiltin,
	};
	await categoryStore.updateCategory(data);
	await onSearch();

	switchLoadMap.value[index] = Object.assign({}, switchLoadMap.value[index], {
		loading: false,
	});
};
